;**********************************************************************
; TITLE : Multiplexer data ENCODER
;**********************************************************************
;	Filename	: MUX_EN10.ASM                                          
;    	Date Started 	: 17/7/98                                               
;	Last edit 	: 30/7/98
;    	File Version	: 1.0
;                                                                     
;	Author		: David L. Jones
;**********************************************************************
;                                                                     
; Files required:
;	p16F84.inc	
;                                                                     
;**********************************************************************
; Notes:            
;	ASSUMING 1MHz XTAL CLOCK WHICH GIVES 4us PER INSTRUCTION
;	
;**********************************************************************

	list      p=16F84             ; list directive to define processor
	#include <p16F84.inc>         ; processor specific variable definitions

	__CONFIG   _CP_OFF & _WDT_ON & _PWRTE_ON & _XT_OSC

;***** VARIABLE DEFINITIONS
	
LOOP1	EQU	0CH		; LOOP VARIABLES
LOOP05	EQU	0DH
LOOP100	EQU	0EH
	
;**********************************************************************
	ORG     0x000		; processor reset vector - start of code
	
;******************************* INITIALISATION STUFF
	CLRF	PORTB		; reset portb
	BSF	STATUS, RP0	; select bank 1
	MOVLW	B'00001111'	; PINS RB0-3=INPUT & RB4-7=OUTPUT
	MOVWF	TRISB
	BCF	OPTION_REG,NOT_RBPU	; ENABLE PULL-UP'S ON PORTB PINS
	BCF	STATUS, RP0	; SELECT BANK 0


;******************************* 
START:
	CLRWDT			; RESET WATCHDOG TIMER
	BCF	PORTB, 4	; SET SERIAL DATA O/P LOW
	CALL	DELAY100	; DELAY 100ms
	BSF	PORTB, 4	; SET SERIAL DATA O/P HIGH (START BIT)
	CALL	DELAY05		; DELAY 0.5ms
;******************************** SEND BIT 0
	BTFSC	PORTB, 0	; CHECK BIT 0 I/P, AND SKIP IF CLEAR
	GOTO	SET0
	BCF	PORTB, 4	; BIT 0 MUST BE LOW, SO SET O/P LOW	
	GOTO 	SKIP0
SET0:	BSF	PORTB, 4	; BIT 0 MUST BE HIGH, SO SET O/P HIGH
SKIP0:	CALL	DELAY1		; DELAY FOR 1ms
;******************************** SEND BIT 1
	BTFSC	PORTB, 1	; CHECK BIT 1 I/P, AND SKIP IF CLEAR
	GOTO	SET1
	BCF	PORTB, 4	; BIT 1 MUST BE LOW, SO SET O/P LOW	
	GOTO 	SKIP1
SET1:	BSF	PORTB, 4	; BIT 1 MUST BE HIGH, SO SET O/P HIGH
SKIP1:	CALL	DELAY1		; DELAY FOR 1ms
;******************************** SEND BIT 2
	BTFSC	PORTB, 2	; CHECK BIT 2 I/P, AND SKIP IF CLEAR
	GOTO	SET2
	BCF	PORTB, 4	; BIT 2 MUST BE LOW, SO SET O/P LOW	
	GOTO 	SKIP2
SET2:	BSF	PORTB, 4	; BIT 2 MUST BE HIGH, SO SET O/P HIGH
SKIP2:	CALL	DELAY1		; DELAY FOR 1ms
;******************************** SEND BIT 3
	BTFSC	PORTB, 3	; CHECK BIT 3 I/P, AND SKIP IF CLEAR
	GOTO	SET3
	BCF	PORTB, 4	; BIT 3 MUST BE LOW, SO SET O/P LOW	
	GOTO 	SKIP3
SET3:	BSF	PORTB, 4	; BIT 3 MUST BE HIGH, SO SET O/P HIGH
SKIP3:	CALL	DELAY1		; DLEAY FOR 1ms

	GOTO 	START		; CONTINUE LOOPING FOR EVER


;******************************** 1ms DELAY SUBROUTINE
DELAY1:
	MOVLW	0x52		; LOOP COUNT VALUE
	MOVWF	LOOP1		; SET LOOP VARIABLE
L1:	DECFSZ	LOOP1,1		; DEC LOOP COUNTER AND SKIP NEXT STATEMENT IF LOOP COUNTER=0
	GOTO 	L1		; CONTINUE LOOPING UNTIL ZERO
	RETURN

;******************************** 0.5ms DELAY SUBROUTINE
DELAY05:
	MOVLW	0x28		; LOOP COUNT VALUE
	MOVWF	LOOP05		; SET LOOP VARIABLE
L05:	DECFSZ	LOOP05,1	; DEC LOOP COUNTER AND SKIP NEXT STATEMENT IF LOOP COUNTER=0
	GOTO 	L05		; CONTINUE LOOPING UNTIL ZERO
	RETURN
	
;******************************** 100ms DELAY SUBROUTINE
DELAY100:
	MOVLW	0x64		; LOOP COUNT VALUE (100)
	MOVWF	LOOP100		; SET LOOP VARIABLE
L100:	DECFSZ	LOOP100,1	; DEC LOOP COUNTER AND SKIP NEXT STATEMENT IF LOOP COUNTER=0
	GOTO 	CALL1		; CONTINUE LOOPING UNTIL ZERO
	GOTO	ENDLOOP		; FINISHED LOOPING
CALL1:	CALL	DELAY1		; DELAY FOR 1ms
	GOTO	L100
ENDLOOP:
	RETURN

	END			; END OF PROGRAM
